// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https ://mozilla.org/MPL/2.0/.

#ifndef DIRECTION_H
#define DIRECTION_H

#include "Angle.h"

namespace Passer {
namespace LinearAlgebra {

struct Vector3;

template <typename T>
class Direction {
 public:
  AngleOf<T> horizontalAngle;
  AngleOf<T> verticalAngle;

  Direction();
  Direction(AngleOf<T> horizontal, AngleOf<T> vertical);
  Direction(Vector3 v);

  const static Direction forward;
  const static Direction back;
  const static Direction up;
  const static Direction down;
  const static Direction left;
  const static Direction right;

  Vector3 ToVector3();
};

}  // namespace LinearAlgebra
}  // namespace Passer
using namespace Passer::LinearAlgebra;

#include <math.h>
#include "Quaternion.h"
#include "Vector3.h"

template <typename T>
Direction<T>::Direction() {
  this->horizontalAngle = AngleOf<T>(0.0f);
  this->verticalAngle = AngleOf<T>(0.0f);
}

template <typename T>
Direction<T>::Direction(AngleOf<T> horizontal, AngleOf<T> vertical) {
  this->horizontalAngle = horizontal;
  this->verticalAngle = vertical;
};

template <typename T>
Direction<T>::Direction(Vector3 v) {
  this->horizontalAngle =
      atan2f(v.Right(), v.Forward()) * Passer::LinearAlgebra::Rad2Deg;
  this->verticalAngle = 90 - acosf(v.Up()) * Passer::LinearAlgebra::Rad2Deg;
}

template <typename T>
Vector3 Direction<T>::ToVector3() {
  Vector3 v = Quaternion::Euler(-(this->verticalAngle.ToFloat()),
                                this->horizontalAngle.ToFloat(), 0) *
              Vector3::forward;
  return v;
}

#endif